/*
 *========================================================================
 * $Id: maxvalue.c,v 1.1 2001/04/20 16:21:13 rgb Exp $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

/*
 *========================================================================
 *  Project work template.
 *========================================================================
 */

#define _ISOC99_SOURCE
#include <math.h>
extern int verbose;

double maxdouble()
{

 double d,dold,dfrac,dnew;
 int i;

 d = 1.0;
 i = 0;

 /*
  * This routine finds the order of infinity (with a factor of two).
  * dold contains the last number that was NOT infinite.
  */
 while(fpclassify(d) != FP_INFINITE){
   if(verbose) printf("d[%d] = %21.16e\n",i,d);
   dold = d;
   d *= 2.0;
   i++;
 }

 /*
  * OK, dold contains the biggest number so far that isn't infinite.
  * We now embark on a binomial search for the boundary number by
  * subdividing d until the fraction added doesn't change the number
  * itself.
  */
 dfrac = dold;
 while(dnew != dold){
   dold = dnew;
   dfrac /= 2.0;
   d = dold + dfrac;
   /* If result isn't infinite, accept it and continue up */
   if(fpclassify(d) != FP_INFINITE) {
     dnew = d;
     i++;
     if(verbose) printf("d[%d] = %21.16e, dold = %21.16e\n",i,d,dold);
   }
 }
 if(verbose) printf("Biggest double that isn't infinite: %21.16e\n",d);

 return(d);

}

float maxfloat()
{

 float d,dold,dfrac,dnew;
 int i;

 d = 1.0;
 i = 0;

 /*
  * This routine finds the order of infinity (with a factor of two).
  * dold contains the last number that was NOT infinite.
  */
 while(fpclassify(d) != FP_INFINITE){
   if(verbose) printf("d[%d] = %21.16e\n",i,d);
   dold = d;
   d *= 2.0;
   i++;
 }

 /*
  * OK, dold contains the biggest number so far that isn't infinite.
  * We now embark on a binomial search for the boundary number by
  * subdividing d until the fraction added doesn't change the number
  * itself.
  */
 dfrac = dold;
 while(dnew != dold){
   dold = dnew;
   dfrac /= 2.0;
   d = dold + dfrac;
   /* If result isn't infinite, accept it and continue up */
   if(fpclassify(d) != FP_INFINITE) {
     dnew = d;
     i++;
     if(verbose) printf("d[%d] = %21.16e, dold = %21.16e\n",i,d,dold);
   }
 }
 if(verbose) printf("Biggest float that isn't infinite: %21.16e\n",d);

 return(d);

}

